#ifndef __ISUBDEVICE__
#define __ISUBDEVICE__

#include "USBAudioObject.h"
#include "AppleiSubEngine.h"
#include "AppleUSBAudioLevelControl.h"
#include "AppleUSBAudioMuteControl.h"

#include <IOKit/IOService.h>
#include <IOKit/IOMessage.h>

class AppleiSubDevice : public IOService {
    OSDeclareDefaultStructors (AppleiSubDevice);

public:
	virtual	bool		init (OSDictionary * properties);
	virtual	void		free (void);
	virtual	bool		start (IOService * provider);
	virtual	void		stop (IOService * provider);
	virtual	bool		terminate (IOOptionBits options = 0);
	virtual	bool		finalize (IOOptionBits options);
};

#endif